-- scope stuff
qualityoflife = qualityoflife or {};
local q = qualityoflife;

q.modules = q.modules or {};
q.modules["QoL_ClassicHeals"] = q.modules["QoL_ClassicHeals"] or {};
local c = q.modules["QoL_ClassicHeals"];
c.name = "QoL_ClassicHeals";
c.version = 11; -- check for toc
c.minCoreVersion = 11;

local framePool = {};

-- patronLib
local patrons = {
	["STAGE1"] = {

	},
	["STAGE2"] = {

	},
	["STAGE3"] = {
		4016349704,
		3338583666
	}
};

-- init
local function UpdateAddon()
	if not QOL_OPTIONS[c.name][q.META_LASTVERSION] or QOL_OPTIONS[c.name][q.META_LASTVERSION] < c.version then
		-- updates here

		QOL_OPTIONS[c.name][q.META_LASTVERSION] = c.version;
	end
	return true;
end

function c:Init()
	if q.initFinished and not c.initFinished and not c.initFailed then
		QOL_OPTIONS = QOL_OPTIONS or {};
		QOL_OPTIONS[c.name] = QOL_OPTIONS[c.name] or {};

		if q.version >= c.minCoreVersion then
			if UpdateAddon() and c:InitOptions() then
				c:HookPlayerFrameEvent();

				QOL_OPTIONS[c.name]["version"] = c.version;

				c.initFinished = true;
				c:Println(q:GetText("Module loaded."));
				return true;
			end
		else
			c:Println(q:GetText("Error while initialization: QoL Core is outdated (version %s)! Minimum required version: %s", q.version, c.minCoreVersion));
			c:Println(q:GetText("Maybe another of your QoL addon comes with an older version. Please use the latest QoL_Core folder or download the latest version from CurseForge."));
		end
		c.initFailed = true;
	end
end

function c:HookPlayerFrameEvent()
	hooksecurefunc("CombatFeedback_OnCombatEvent", function(self, event, flags, amount)
		if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PLAYER][c.OPT_UIATTR_ENABLED] and event == "HEAL" then
			self.feedbackText:Hide();
		elseif not self.feedbackText:IsVisible() then
			self.feedbackText:Show();
		end
	end);
end

-- event handling
local eventFrame;
local function HookEvents()
	if not eventFrame then
		eventFrame = q:GetEventFrame();
		eventFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		eventFrame:HookScript("OnEvent", function(self, event, arg1, arg2, arg3, arg4)
			if event then
				if event == "ADDON_LOADED" and arg1 == c.name then
					c:Init();
				elseif event == "PLAYER_ENTERING_WORLD" and c.initFinished then
					if not c.profilesInitialized then
						c:InitProfiles(true);
						c.profilesInitialized = true;
					end

					if q.modules["QoL_PatronLib"] then
						q.modules["QoL_PatronLib"]:RegisterAddon("ClassicHeals", patrons);
					end
				elseif event == "PLAYER_TARGET_CHANGED" and c.initFinished then
					for index, frame in ipairs(framePool) do
						if frame.isBusy and frame.profile == c.PROFILE_TARGET then
							UIFrameFadeOut(frame, 0, 1, 0);
						end
					end
				elseif event == "COMBAT_LOG_EVENT_UNFILTERED" and c.initFinished then
					c:OnEvent(event, CombatLogGetCurrentEventInfo());
				end
			end
		end);
	end
end
HookEvents();

function c:OnEvent(event, ...)
	local timestamp, subevent, _, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags = ...;

	if (sourceName == q:GetCharName() or destName == q:GetCharName()) and (subevent == "SPELL_HEAL" or subevent == "SPELL_PERIODIC_HEAL") then
		local spellId, spellName, spellSchool, amount, overhealing, absorbed, critical = select(12, ...);
		c:DisplayHeal(destName, amount, overhealing, critical);
	end
end

function c:DisplayHeal(target, amount, overhealing, critical)
	if target then
		local targetFramesByProfile = c:GetTargetFrames(target);

		for profile, targetFramesBySubProfile in pairs(targetFramesByProfile) do
			for subProfile, targetFrames in pairs(targetFramesBySubProfile) do
				for index, targetFrame in ipairs(targetFrames) do
					local frame, message = c:GetFrame(targetFrame);
					frame.isBusy = true;
					frame.profile = profile;
					frame.subProfile = subProfile;

					if overhealing > 0 then
						local totalAmount = amount - overhealing;
						if QOL_OPTIONS[c.name][profile][subProfile][c.OPT_UIATTR_SHOWOVERHEAL] then
							message = "+" .. totalAmount .. " |c" .. q:GetColorHexFromRGBA(QOL_OPTIONS[c.name][profile][subProfile][c.OPT_UIATTR_FONTCOLOR_OVERHEAL]) .. "(" .. overhealing .. ")";
						else
							if totalAmount > 0 then
								message = "+" .. totalAmount;
							else
								message = "|c" .. q:GetColorHexFromRGBA(QOL_OPTIONS[c.name][profile][subProfile][c.OPT_UIATTR_FONTCOLOR_OVERHEAL]) .. q:GetText("full");
							end
						end
					else
						message = "+" .. amount;
					end

					frame:SetParent(targetFrame);
					frame:SetFrameStrata("HIGH");
					c:SetFont(frame, critical);
					c:SetFramePosition(frame);
					frame:SetText(message);

					if critical then
						UIFrameFadeIn(frame, 0.1, 0, 1);
						frame.fadeInfo.finishedFunc = function()
							UIFrameFadeOut(frame, 3, 3, 0);
							frame.fadeInfo.finishedFunc = function()
								frame.isBusy = false;
							end
						end
					else
						UIFrameFadeIn(frame, 0.3, 0, 1);
						frame.fadeInfo.finishedFunc = function()
							UIFrameFadeOut(frame, 1.5, 1.5, 0);
							frame.fadeInfo.finishedFunc = function()
								frame.isBusy = false;
							end
						end
					end
				end
			end
		end
	end
end

function c:GetFrame(parent)
	for index, frame in ipairs(framePool) do
		if not frame.isBusy then
			return frame;
		end
	end

	local newFrame = CreateFrame("Frame", "ClassicHealsFrame" .. (table.getn(framePool) + 1), parent);
	newFrame:SetWidth(160);
	newFrame:SetHeight(32);
	newFrame.fontString = newFrame:CreateFontString(newFrame:GetName() .. "FontString", "ARTWORK", "GameFontNormal");
	newFrame.fontString:SetPoint("CENTER");

	function newFrame.SetText(self, text)
		self.fontString:SetText(text);
	end

	q:HandleElvUi(newFrame, function()
		newFrame:SetBackdropColor(0, 0, 0, 0);
		newFrame:SetBackdropBorderColor(0, 0, 0, 0);
	end);

	table.insert(framePool, newFrame);
	return newFrame;
end

function c:SetFramePosition(frame)
	frame:ClearAllPoints();
	frame:SetPoint("CENTER", QOL_OPTIONS[c.name][frame.profile][frame.subProfile][c.OPT_UIATTR_XOFFSET], QOL_OPTIONS[c.name][frame.profile][frame.subProfile][c.OPT_UIATTR_YOFFSET]);
end

function c:SetFont(frame, critical)
	local size, color;

	if critical then
		size = QOL_OPTIONS[c.name][frame.profile][frame.subProfile][c.OPT_UIATTR_FONTSIZE_CRITICAL];
		color = QOL_OPTIONS[c.name][frame.profile][frame.subProfile][c.OPT_UIATTR_FONTCOLOR_CRITICAL];
	else
		size = QOL_OPTIONS[c.name][frame.profile][frame.subProfile][c.OPT_UIATTR_FONTSIZE];
		color = QOL_OPTIONS[c.name][frame.profile][frame.subProfile][c.OPT_UIATTR_FONTCOLOR];
	end

	if size then
		if frame.subProfile == c.PROFILE_RAID_COMPACT then
			size = size * min(DefaultCompactUnitFrameSetupOptions.height / 36, DefaultCompactUnitFrameSetupOptions.width / 72) -- thx to rulezyx
		end
		frame.fontString:SetFont(frame.fontString:GetFont(), size, QOL_OPTIONS[c.name][frame.profile][frame.subProfile][c.OPT_UIATTR_FONTFLAGS]);
	end

	if color then
		frame.fontString:SetTextColor(q:GetColorRBGA(color));
	end
end

function c:Println(str)
	if str then
		q:Println(str, "ClassicHeals");
	end
end